package com.linuxnet.jpcsc;


/**
 * Warning, Error, Parameter Codes.
 */
public final class PCSC{
    /**
     * Name of the native library we depend on.
     */
    public static final String pcscNativeLibName = "jpcsc";

    /**
     * Note that PCSClite versions up to 0.8.2 return 1 instead of 0 on success
     * and thus do not match jpcsc.
     */
    public final static int SUCCESS;

    /** Scope in user space */
    public static final int SCOPE_USER;
    /** Scope in terminal   */
    public static final int SCOPE_TERMINAL;
    /** Scope in system     */
    public static final int SCOPE_SYSTEM;
    /** Scope is global, mapped on SCOPE_SYSTEM on Windows */
    public static final int SCOPE_GLOBAL;

    /** T=0 active protocol. */
    public static final int PROTOCOL_T0;
    /** T=1 active protocol. */
    public static final int PROTOCOL_T1;
    /** Raw active protocol. */
    public static final int PROTOCOL_RAW;
    /** IFD determines protocol. */
    public static final int PROTOCOL_ANY;
    /** IFD determines protocol. */
    public static final int PROTOCOL_DEFAULT;


    /** Exclusive mode only  */
    public static final int SHARE_EXCLUSIVE;
    /** Shared mode only     */
    public static final int SHARE_SHARED;
    /** Raw mode only */
    public static final int SHARE_DIRECT;


    /** Do nothing on close  */
    public static final int LEAVE_CARD;
    /** Reset on close       */
    public static final int RESET_CARD;
    /** Power down on close  */
    public static final int UNPOWER_CARD;
    /** Eject on close       */
    public static final int EJECT_CARD;

    /** Unknown state        */
    public static final int UNKNOWN;
    /** Card is absent       */
    public static final int ABSENT;
    /** Card is present      */
    public static final int PRESENT;
    /** Card not powered     */
    public static final int SWALLOWED;
    /** Card is powered      */
    public static final int POWERED;
    /** Ready for PTS        */
    public static final int NEGOTIABLE;
    /** PTS has been set     */
    public static final int SPECIFIC;

    /** App wants status     */
    public static final int STATE_UNAWARE;
    /** Ignore this reader   */
    public static final int STATE_IGNORE;
    /** State has changed    */
    public static final int STATE_CHANGED;
    /** Reader unknown       */
    public static final int STATE_UNKNOWN;
    /** Status unavailable   */
    public static final int STATE_UNAVAILABLE;
    /** Card removed         */
    public static final int STATE_EMPTY;
    /** Card inserted        */
    public static final int STATE_PRESENT;
    /** ATR matches card 	*/
    public static final int STATE_ATRMATCH;
    /** Exclusive Mode       */
    public static final int STATE_EXCLUSIVE;
    /** Shared Mode          */
    public static final int STATE_INUSE;
    /** Unresponsive card    */
    public static final int STATE_MUTE;


    public static final int E_CANCELLED;
    public static final int E_CANT_DISPOSE;
    public static final int E_INSUFFICIENT_BUFFER;
    public static final int E_INVALID_ATR;
    public static final int E_INVALID_HANDLE;
    public static final int E_INVALID_PARAMETER;
    public static final int E_INVALID_TARGET;
    public static final int E_INVALID_VALUE;
    public static final int E_NO_MEMORY;
    public static final int F_COMM_ERROR;
    public static final int F_INTERNAL_ERROR;
    public static final int F_UNKNOWN_ERROR;
    public static final int F_WAITED_TOO_LONG;
    public static final int E_UNKNOWN_READER;
    public static final int E_TIMEOUT;
    public static final int E_SHARING_VIOLATION;
    public static final int E_NO_SMARTCARD;
    public static final int E_UNKNOWN_CARD;
    public static final int E_PROTO_MISMATCH;
    public static final int E_NOT_READY;
    public static final int E_SYSTEM_CANCELLED;
    public static final int E_NOT_TRANSACTED;
    public static final int E_READER_UNAVAILABLE;

    public static final int W_UNSUPPORTED_CARD;
    public static final int W_UNRESPONSIVE_CARD;
    public static final int W_UNPOWERED_CARD;
    public static final int W_RESET_CARD;
    public static final int W_REMOVED_CARD;

    public static final int E_PCI_TOO_SMALL;
    public static final int E_READER_UNSUPPORTED;
    public static final int E_DUPLICATE_READER;
    public static final int E_CARD_UNSUPPORTED;
    public static final int E_NO_SERVICE;
    public static final int E_SERVICE_STOPPED;

    private static final int CODES_COUNT = 70;

    static{
        Runtime.getRuntime().loadLibrary(PCSC.pcscNativeLibName);

        int[] codes = null;
        codes = NativeInitialize(Context.class, Card.class, State.class);

        SUCCESS = codes[0];

        SCOPE_USER = codes[1];
        SCOPE_TERMINAL = codes[2];
        SCOPE_SYSTEM = codes[3];
        SCOPE_GLOBAL = codes[4];

        PROTOCOL_T0 = codes[5];
        PROTOCOL_T1 = codes[6];
        PROTOCOL_RAW = codes[7];
        PROTOCOL_ANY = codes[8];
        PROTOCOL_DEFAULT = PROTOCOL_ANY;

        SHARE_EXCLUSIVE = codes[9];
        SHARE_SHARED = codes[10];
        SHARE_DIRECT = codes[11];

        LEAVE_CARD = codes[12];
        RESET_CARD = codes[13];
        UNPOWER_CARD = codes[14];
        EJECT_CARD = codes[15];

        UNKNOWN = codes[16];
        ABSENT = codes[17];
        PRESENT = codes[18];
        SWALLOWED = codes[19];
        POWERED = codes[20];
        NEGOTIABLE = codes[21];
        SPECIFIC = codes[22];

        STATE_UNAWARE = codes[23];
        STATE_IGNORE = codes[24];
        STATE_CHANGED = codes[25];
        STATE_UNKNOWN = codes[26];
        STATE_UNAVAILABLE = codes[27];
        STATE_EMPTY = codes[28];
        STATE_PRESENT = codes[29];
        STATE_ATRMATCH = codes[30];
        STATE_EXCLUSIVE = codes[31];
        STATE_INUSE = codes[32];
        STATE_MUTE = codes[33];

        E_CANCELLED = codes[34];
        E_CANT_DISPOSE = codes[35];
        E_INSUFFICIENT_BUFFER = codes[36];
        E_INVALID_ATR = codes[37];
        E_INVALID_HANDLE = codes[38];
        E_INVALID_PARAMETER = codes[39];
        E_INVALID_TARGET = codes[40];
        E_INVALID_VALUE = codes[41];
        E_NO_MEMORY = codes[42];
        F_COMM_ERROR = codes[43];
        F_INTERNAL_ERROR = codes[44];
        F_UNKNOWN_ERROR = codes[45];
        F_WAITED_TOO_LONG = codes[46];
        E_UNKNOWN_READER = codes[47];
        E_TIMEOUT = codes[48];
        E_SHARING_VIOLATION = codes[49];
        E_NO_SMARTCARD = codes[50];
        E_UNKNOWN_CARD = codes[51];
        E_PROTO_MISMATCH = codes[52];
        E_NOT_READY = codes[53];
        E_SYSTEM_CANCELLED = codes[54];
        E_NOT_TRANSACTED = codes[55];
        E_READER_UNAVAILABLE = codes[56];

        W_UNSUPPORTED_CARD = codes[57];
        W_UNRESPONSIVE_CARD = codes[58];
        W_UNPOWERED_CARD = codes[59];
        W_RESET_CARD = codes[60];
        W_REMOVED_CARD = codes[61];

        E_PCI_TOO_SMALL = codes[62];
        E_READER_UNSUPPORTED = codes[63];
        E_DUPLICATE_READER = codes[64];
        E_CARD_UNSUPPORTED = codes[65];
        E_NO_SERVICE = codes[66];
        E_SERVICE_STOPPED = codes[67];
    }


    static void initialize(){
        // all initialization takes place in the static initializer so far ...
        return;
    }

    static native int[] NativeInitialize(Class ctxClass, Class cardClass, Class stateClass);
}
