package be.godot.sc.engine;

import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Frame;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.security.interfaces.RSAPublicKey;

import be.godot.sc.exceptions.InvalidPinException;
import be.godot.sc.exceptions.InvalidResponse;
import be.godot.sc.exceptions.NoReadersAvailable;
import be.godot.sc.exceptions.NoSuchFeature;
import be.godot.sc.exceptions.PinException;
import be.godot.sc.exceptions.SignatureGenerationException;
import be.godot.sc.exceptions.SmartCardReaderException;
import be.godot.sc.exceptions.UnknownCardException;
import be.godot.sc.gui.PinPad;
import be.godot.sc.interfaces.BelpicCommandsEngine;
import be.godot.sc.interfaces.EidCardInterface;
import be.godot.util.LoadLibraryFromJar;
import be.godot.util.MathUtils;
import be.godot.util.TextUtils;
import be.godot.util.TimeUtils;

import com.linuxnet.jpcsc.PCSC;

public class EidCard extends SmartCard implements EidCardInterface,
		BelpicCommandsEngine {
	public static byte[] selectMasterFile = { (byte) 0x00, (byte) 0xa4,
			(byte) 0x02, (byte) 0x00, (byte) 0x02, (byte) 0x3f, (byte) 0x00 };

	private int myType = EidCardInterface.GENERIC_EID_CARD;

	public EidCard(int type, String appName) {
		LoadLibraryFromJar.initLocalLibrary(PCSC.pcscNativeLibName, appName);
		myType = type;
	}

	public int returnMyType() {
		return myType;
	}

	public RSAPublicKey getPublicAuthenticationKey() throws NoSuchFeature {
		RSAPublicKey publicKey = null;
		try {
			publicKey = (RSAPublicKey) getAuthenticationCertificate()
					.getPublicKey();
		} catch (Exception e) {
			e.printStackTrace();
			throw new NoSuchFeature();
		}
		return publicKey;
	}

	public RSAPublicKey getPublicNonRepudiationKey() throws NoSuchFeature {
		RSAPublicKey publicKey = null;
		try {
			publicKey = (RSAPublicKey) getNonRepudiationCertificate()
					.getPublicKey();
		} catch (Exception e) {
			e.printStackTrace();
			throw new NoSuchFeature();
		}
		return publicKey;
	}

	public X509Certificate getAuthenticationCertificate() throws NoSuchFeature {
		if (authenticationCertificate == null)
			try {
				lookForSmartCard();
				InputStream inStream = new ByteArrayInputStream(
						getAuthCertificateBytes());
				CertificateFactory cf = CertificateFactory.getInstance("X.509");
				authenticationCertificate = (X509Certificate) cf
						.generateCertificate(inStream);
				inStream.close();
			} catch (Exception e) {
				e.printStackTrace();
				throw new NoSuchFeature();
			}
		return authenticationCertificate;
	}

	public X509Certificate getCertificationAuthorityCertificate()
			throws NoSuchFeature {
		if (caCertificate == null)
			try {
				lookForSmartCard();
				InputStream inStream = new ByteArrayInputStream(
						readCACertificateBytes());
				CertificateFactory cf = CertificateFactory.getInstance("X.509");
				X509Certificate cert = (X509Certificate) cf
						.generateCertificate(inStream);
				inStream.close();

			} catch (Exception e) {
				e.printStackTrace();
				throw new NoSuchFeature();
			}
		return caCertificate;
	}

	public X509Certificate getRootCertificationAuthorityCertificate()
			throws NoSuchFeature {
		if (rootCACertificate == null)
			try {
				lookForSmartCard();
				InputStream inStream = new ByteArrayInputStream(this
						.readRootCACertificateBytes());
				CertificateFactory cf = CertificateFactory.getInstance("X.509");
				rootCACertificate = (X509Certificate) cf
						.generateCertificate(inStream);
				inStream.close();
			} catch (Exception e) {
				e.printStackTrace();
				throw new NoSuchFeature();
			}
		return rootCACertificate;
	}

	public X509Certificate getNonRepudiationCertificate() throws NoSuchFeature {
		if (nonRepudiationCertificate == null)
			try {
				lookForSmartCard();
				InputStream inStream = new ByteArrayInputStream(
						getNonRepCertificateBytes());
				CertificateFactory cf = CertificateFactory.getInstance("X.509");
				nonRepudiationCertificate = (X509Certificate) cf
						.generateCertificate(inStream);
				inStream.close();
			} catch (Exception e) {
				e.printStackTrace();
				throw new NoSuchFeature();
			}
		return nonRepudiationCertificate;
	}

	public byte[] getAuthCertificateBytes() throws NoSuchFeature {
		if (authenticationCertificateBytes == null)
			try {
				lookForSmartCard();
				authenticationCertificateBytes = readBinaryFile(selectAuthenticationCertificateCommand);

			} catch (Exception e) {
				e.printStackTrace();
				throw new NoSuchFeature();
			}
		return authenticationCertificateBytes;
	}

	public byte[] getNonRepCertificateBytes() throws NoSuchFeature {
		if (nonRepudiationCertificateBytes == null)
			try {
				lookForSmartCard();
				nonRepudiationCertificateBytes = readBinaryFile(selectNonRepudiationCertificateCommand);

			} catch (Exception e) {
				e.printStackTrace();
				throw new NoSuchFeature();
			}
		return nonRepudiationCertificateBytes;
	}

	public byte[] readCACertificateBytes() throws NoSuchFeature {
		try {
			if (caCertificateBytes == null) {
				lookForSmartCard();
				caCertificateBytes = readBinaryFile(selectCaCertificateCommand);
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new NoSuchFeature();
		}
		return caCertificateBytes;
	}

	public byte[] readIdentityFileSignatureBytes() throws NoSuchFeature {
		try {
			if (identityFileSignatureBytes == null) {
				lookForSmartCard();
				identityFileSignatureBytes = readBinaryFile(selectIdentityFileSignatureCommand);
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new NoSuchFeature();
		}
		return identityFileSignatureBytes;
	}

	public byte[] readAddressFileSignatureBytes() throws NoSuchFeature {
		try {
			if (addressFileSignatureBytes == null) {
				lookForSmartCard();
				addressFileSignatureBytes = readBinaryFile(selectAddressFileSignatureCommand);
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new NoSuchFeature();
		}
		return addressFileSignatureBytes;
	}

	public byte[] readRRNCertificateBytes() throws NoSuchFeature {
		try {
			if (rrnCertificateBytes == null) {
				lookForSmartCard();
				rrnCertificateBytes = readBinaryFile(selectRrnCertificateCommand);
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new NoSuchFeature();
		}
		return rrnCertificateBytes;
	}

	public byte[] readRootCACertificateBytes() throws UnknownCardException,
			NoReadersAvailable, NoSuchFeature, SmartCardReaderException,
			InvalidResponse {
		if (rootCACertificateBytes == null) {
			lookForSmartCard();
			rootCACertificateBytes = readBinaryFile(selectRootCaCertificateCommand);
		}
		return rootCACertificateBytes;
	}

	public byte[] readCitizenIdentityDataBytes() throws UnknownCardException,
			NoReadersAvailable, NoSuchFeature, SmartCardReaderException,
			InvalidResponse {
		if (citizenIdentityFileBytes == null) {
			lookForSmartCard();
			citizenIdentityFileBytes = readBinaryFile(selectCitizenIdentityDataCommand);
		}
		return citizenIdentityFileBytes;
	}

	public byte[] readCitizenAddressBytes() throws UnknownCardException,
			NoReadersAvailable, NoSuchFeature, SmartCardReaderException,
			InvalidResponse {
		if (citizenAddressBytes == null) {
			lookForSmartCard();
			citizenAddressBytes = readBinaryFile(selectCitizenAddressDataCommand);
		}
		return citizenAddressBytes;
	}

	public byte[] readCitizenPhotoBytes() throws NoReadersAvailable,
			InvalidResponse {
		lookForSmartCard();
		if (citizenPhoto == null) {
			citizenPhoto = readBinaryFile(selectCitizenPhotoCommand);
		}
		return citizenPhoto;
	}

	public byte[] generateSignature(byte[] preparationCommand,
			byte[] signatureGenerationCommand, byte[] datahash)
			throws NoSuchFeature, InvalidResponse, NoReadersAvailable,
			SignatureGenerationException {
		lookForSmartCard();
		SmartCardResponse scr = null;
		try {
			myReader.sendCommand(preparationCommand);
			pinValidationEngine();
			byte[] apdu = new byte[signatureGenerationCommand.length];
			for (int i = 0; i < signatureGenerationCommand.length; i++)
				apdu[i] = signatureGenerationCommand[i];
			for (int i = 0; i < MathUtils.min(20, datahash.length); i++)
				apdu[i + 5] = datahash[i];
			scr = new SmartCardResponse(myReader.sendCommand(apdu));
		} catch (Exception e) {
			e.printStackTrace();
		}
		if (scr.getCommandStatus().equals("9000"))
			return scr.getData();
		if (scr.getCommandStatus().equals("6180")) {
			scr = new SmartCardResponse(retrieveSignatureBytes());
			if (scr.getCommandStatus().equals("9000"))
				return scr.getData();
		}
		throw new SignatureGenerationException();
	}

	private String signatureSessionText;

	public final static String authenticationSignatureSession = "Authentication session";

	public final static String nonRepudiationSignatureSession = "Non-Repudiation session";

	public byte[] generateAuthenticationSignature(byte[] datahash)
			throws NoSuchFeature, NoReadersAvailable, InvalidResponse,
			SignatureGenerationException {
		signatureSessionText = authenticationSignatureSession;
		return generateSignature(prepareForAuthenticationSignatureCommand,
				generateSignatureCommand, datahash);
	}

	public byte[] generateNonRepudiationSignature(byte[] datahash)
			throws NoSuchFeature, NoReadersAvailable, InvalidResponse,
			SignatureGenerationException {
		signatureSessionText =nonRepudiationSignatureSession;
		return generateSignature(prepareForNonRepudiationSignatureCommand,
				generateSignatureCommand, datahash);
	}

	private byte[] citizenPhoto = null;

	private byte[] citizenAddressBytes = null;

	private byte[] rootCACertificateBytes = null;

	private byte[] citizenIdentityFileBytes = null;

	private byte[] authenticationCertificateBytes = null;

	private byte[] nonRepudiationCertificateBytes = null;

	private byte[] caCertificateBytes = null;

	private byte[] rrnCertificateBytes = null;

	private byte[] identityFileSignatureBytes = null;

	private X509Certificate rootCACertificate = null;

	private X509Certificate caCertificate = null;

	private X509Certificate nonRepudiationCertificate = null;

	private X509Certificate authenticationCertificate = null;

	private byte[] addressFileSignatureBytes = null;

	public void clearCache() {
		citizenPhoto = null;
		citizenAddressBytes = null;
		rootCACertificateBytes = null;
		citizenIdentityFileBytes = null;
		authenticationCertificateBytes = null;
		nonRepudiationCertificateBytes = null;
		authenticationCertificate = null;
		caCertificateBytes = null;
		rrnCertificateBytes = null;
		identityFileSignatureBytes = null;
		rootCACertificate = null;
		caCertificate = null;
		nonRepudiationCertificate = null;
		addressFileSignatureBytes = null;
	}

	private String defaultPin = "";

	public void setPin(String pinValue) {
		defaultPin = pinValue;
	}

	public void pinValidationEngine() throws InvalidPinException,
			InvalidResponse, PinException, NoSuchFeature, NoReadersAvailable {
		pinValidationEngine(defaultPin, "");
	}

	private byte[] verifyThisPin(String pinvalue) throws InvalidResponse {
		return myReader.sendCommand(insertPinIntoApdu(verifyPinApdu, pinvalue));
	}

	private boolean checkThisPin(String pinvalue, PinPad pinPad,
			int nofMilliSecondsBeforeReturning) throws InvalidResponse {
		boolean keepTrying = true;
		byte[] result = verifyThisPin(pinvalue);
		if (TextUtils.hexDump(result).equals("9000")) {
			pinPad.setStatusText("OK...");
			TimeUtils.sleep(nofMilliSecondsBeforeReturning);
			keepTrying = false;
		} else {
			if (TextUtils.hexDump(result).equals("63C2")) {
				pinPad.setStatusText("Invalid PIN, 2 tries left...");
				TimeUtils.sleep(nofMilliSecondsBeforeReturning);
			} else if (TextUtils.hexDump(result).equals("63C1")) {
				pinPad.setStatusText("Invalid PIN, 1 try left...");
				TimeUtils.sleep(nofMilliSecondsBeforeReturning);
			} else {
				if (TextUtils.hexDump(result).equals("63C0")) {
					pinPad.setStatusText("Invalid PIN, Card blocked...");
					TimeUtils.sleep(nofMilliSecondsBeforeReturning);
					keepTrying = false;
				}
			}
		}
		return keepTrying;
	}

	private final static int pinPadDelayBeforeReturning = 500;

	private void pinValidationEngine(String pinvalue, String pintext)
			throws InvalidPinException, PinException, InvalidResponse,
			NoSuchFeature, NoReadersAvailable {
		lookForSmartCard();
		Frame window = null;
		PinPad pinPad = null;
		if (myReader.getMyName().toUpperCase().indexOf(
				VascoSmartCardReader.referenceName.toUpperCase()) >= 0) {
			myReader.verifyPIN();
			return;
		} else if (pinvalue != "")
			checkThisPin(pinvalue, pinPad, pinPadDelayBeforeReturning);
		else {
			try {
				pinPad = new PinPad(signatureSessionText,
						EidCardInterface.minNofPinDigits,
						EidCardInterface.maxNofPinDigits, pintext);
				window = new Frame();
				window.setTitle("Pseudo-secure PinPad");
				window.setLayout(new FlowLayout());
				window.setFont(new Font("Helvetica", Font.PLAIN, 12));
				window.setBackground(Color.white);
				window.add(pinPad);
				window.pack();
				window.setVisible(true);
				window.show();
				do {
					while (pinPad.stillBusy())
						TimeUtils.sleep(100);
					if (pinPad.cancelPressed())
						throw new PinException("User cancelled PIN entry");
					else
						pinvalue = pinPad.getPinValue();
					pinPad.setStatusText("Processing Verify PIN command...");
				} while (checkThisPin(pinvalue, pinPad,
						pinPadDelayBeforeReturning));
			} catch (PinException e) {
				e.printStackTrace();
				throw e;
			} finally {
				if (window != null) {
					window.setVisible(false);
					window.dispose();
					window = null;
				}
			}
		}
	}

	public void changeThisPin(String currentpinvalue, String newpinvalue,
			PinPad pinPad, int nofMilliSecondsBeforeReturning)
			throws InvalidResponse {
		byte[] result = myReader.sendCommand(insertTwoPinsIntoApdu(
				changePinApdu, currentpinvalue, newpinvalue));
		if (TextUtils.hexDump(result).equals("9000")) {
			pinPad.setStatusText("OK...");
			TimeUtils.sleep(nofMilliSecondsBeforeReturning);
		} else {
			if (TextUtils.hexDump(result).equals("63C2")) {
				pinPad.setStatusText("Invalid PIN, 2 tries left...");
				TimeUtils.sleep(nofMilliSecondsBeforeReturning);
			} else if (TextUtils.hexDump(result).equals("63C1")) {
				pinPad.setStatusText("Invalid PIN, 1 try left...");
				TimeUtils.sleep(nofMilliSecondsBeforeReturning);
			} else {
				if (TextUtils.hexDump(result).equals("63C0")) {
					pinPad.setStatusText("Invalid PIN, Card blocked...");
					TimeUtils.sleep(nofMilliSecondsBeforeReturning);
				}
			}
		}
	}

	public void changePin() throws InvalidPinException, PinException,
			InvalidResponse, NoSuchFeature, NoReadersAvailable {
		lookForSmartCard();
		if (myReader.getMyName().toUpperCase().indexOf(
				VascoSmartCardReader.referenceName.toUpperCase()) >= 0) {
			myReader.changePIN();
			return;
		} else {
			Frame window = null;
			String currentPin = "";
			String newPin = "";
			PinPad pinPad = null;
			if (myReader.getMyName().toUpperCase().indexOf(
					VascoSmartCardReader.referenceName.toUpperCase()) >= 0) {
				myReader.verifyPIN();
				return;
			} else {
				try {
					pinPad = new PinPad(signatureSessionText,EidCardInterface.minNofPinDigits,
							EidCardInterface.maxNofPinDigits, "current");
					window = new Frame();
					window.setTitle("Pseudo-secure PinPad");
					window.setLayout(new FlowLayout());
					window.setFont(new Font("Helvetica", Font.PLAIN, 12));
					window.setBackground(Color.white);
					window.add(pinPad);
					window.pack();
					window.setVisible(true);
					window.show();
					while (pinPad.stillBusy())
						TimeUtils.sleep(100);
					if (pinPad.cancelPressed())
						throw new PinException("User cancelled PIN entry");
					else
						currentPin = pinPad.getPinValue();
					window.remove(pinPad);
					pinPad = new PinPad(signatureSessionText,EidCardInterface.minNofPinDigits,
							EidCardInterface.maxNofPinDigits, "new");
					window.add(pinPad);
					window.pack();
					window.setVisible(true);
					window.show();
					while (pinPad.stillBusy())
						TimeUtils.sleep(100);
					if (pinPad.cancelPressed())
						throw new PinException("User cancelled PIN entry");
					else
						newPin = pinPad.getPinValue();
					pinPad.setStatusText("Processing Change PIN command...");
					changeThisPin(currentPin, newPin, pinPad,
							5 * pinPadDelayBeforeReturning);
				} catch (PinException e) {
					e.printStackTrace();
					throw e;
				} finally {
					if (window != null) {
						window.setVisible(false);
						window.dispose();
						window = null;
					}
				}
			}
		}
	}

	public final static int ApduHeaderLength = 5;

	public final static int PinBlockLength = 8;

	private byte[] insertPinIntoApdu(byte[] apdu, String pinvalue) {
		return insertPinIntoApdu(apdu, ApduHeaderLength, pinvalue);
	}

	private byte[] insertTwoPinsIntoApdu(byte[] apdu, String pin1, String pin2) {
		byte[] tmpApdu = insertPinIntoApdu(apdu, ApduHeaderLength, pin1);
		return insertPinIntoApdu(tmpApdu, ApduHeaderLength + PinBlockLength,
				pin2);
	}

	private byte[] insertPinIntoApdu(byte[] apdu, int pinBlockOffset,
			String pinvalue) {
		String pinValue = "";
		if (2 * ((int) (pinvalue.length() / 2)) == pinvalue.length())
			pinValue = pinvalue;
		else
			pinValue = pinvalue + "F";
		byte[] newApdu = new byte[apdu.length];
		for (int i = 0; i < newApdu.length; i++)
			newApdu[i] = apdu[i];
		int offsetInCommand = pinBlockOffset;
		newApdu[offsetInCommand++] = (byte) (2 * 16 + pinValue.length());
		for (int i = 0; i < pinValue.length(); i += 2) {
			newApdu[offsetInCommand++] = (byte) (Integer.parseInt(pinValue
					.substring(i, i + 2), 16));
		}
		return newApdu;
	}

	public void reActivate(String puk1, String puk2)
			throws InvalidPinException, InvalidResponse, NoSuchFeature,
			NoReadersAvailable {
		lookForSmartCard();
		String pukValue = puk2 + puk1;
		byte[] unblockCardCommand = insertPinIntoApdu(unblockCardApdu, pukValue);
		byte[] result = myReader.sendCommand(unblockCardCommand);
	}

	public byte[] selectFile(int fileid) throws UnknownCardException,
			NoSuchFeature, SmartCardReaderException, InvalidResponse,
			NoReadersAvailable {
		lookForSmartCard();
		return myReader.sendCommand(selectFileCommand);
	}

	public byte[] retrieveSignatureBytes() throws NoSuchFeature,
			NoReadersAvailable, InvalidResponse {
		lookForSmartCard();
		return myReader.sendCommand(retrieveSignatureCommand);
	}

	public String nameOfActiveReader() throws NoReadersAvailable {
		lookForSmartCard();
		return myReader.getMyName();
	}

	public byte[] fetchATR() throws NoReadersAvailable {
		lookForSmartCard();
		return myReader.getATR();
	}

	public SmartCardReader getSmartCardReader() throws NoReadersAvailable {
		lookForSmartCard();
		return myReader;
	}

	private void lookForSmartCard() throws NoReadersAvailable {
		if (smartCardReaderShouldBeInitialized)
			lookForSmartCard(defaultPreferredSmartCardReader,
					defaultTimeoutInMilliSecondsBeforeTryingAnotherReader);
	}

	private boolean smartCardReaderShouldBeInitialized = true;

	private void lookForSmartCard(String preferredReader,
			int milliSecondsBeforeTryingAnotherReader)
			throws NoReadersAvailable {
		if (smartCardReaderShouldBeInitialized) {
			myReader = new SmartCardReader();
			myReader.lookForSmartCard(preferredReader,
					milliSecondsBeforeTryingAnotherReader);
			smartCardReaderShouldBeInitialized = false;
		}
	}
}