package be.godot.util.testSuite;

import java.math.BigInteger;

import be.godot.sc.exceptions.UnsupportedEncodingException;
public class TextUtils {

	private final static String nullString="null";
	
	public static String hexDump(BigInteger big) {
		if (big == null)
			return nullString;
		else
			return hexDump(big.toByteArray());
	}

	public static String hexDump(byte[] data) {
		if (data == null)
			return nullString;
		else
		return hexDump(data, 0, data.length);
	}

	public static String hexDump(byte[] data, int length) {
		if (data == null)
			return nullString;
		else
		return hexDump(data, 0, length);
	}

	public static String hexDump(byte[] data, int offset, int length) {
		if (data == null)
			return nullString;
		else
		return ba2s(data, offset, length);
	}

	static void print(byte[] response) {
		print(response, 0, response.length);
	}

	static void printInterpreted(byte[] response) {
		print(response, 0, response.length - 2);
		System.err.println("Status words  "
				+ ba2s(response, response.length - 2, 2, defaultFormat));
	}

	static void printNonInterpreted(String prefix, byte[] response,
			String postfix) {
		System.err.println(prefix
				+ ba2s(response, 0, response.length, defaultFormat) + postfix);
	}

	static void printInterpreted(String prefix, byte[] response, String postfix) {
		System.err.println(prefix
				+ ba2s(response, 0, response.length - 2, defaultFormat)
				+ postfix);
		System.err.println("Status words  "
				+ ba2s(response, response.length - 2, 2, defaultFormat));
	}

	static void print(byte[] response, int off, int length) {
		System.err.println("Response  "
				+ ba2s(response, off, length, defaultFormat));
	}

	final static public int tagLen = 1;

	public static int getTagIdentifier(byte[] data, int pos) {
		return data[pos];
	}

	public static int getDataLen(byte[] data, int pos) {
		int i = 0;
		int len = 0;
		while (MathUtils.unsignedInt(data[pos + i]) == 255) {
			len = len * 255 + 255;
			i++;
		}
		len += MathUtils.unsignedInt(data[pos + i]);
		return len;
	}

	public static int getLenLen(byte[] data, int pos) {
		int i = 0;
		while (MathUtils.unsignedInt(data[pos + i]) == 255) {
			i++;
		}
		return i + 1;
	}

	public static String getString(byte[] data, int pos, int len)
		throws UnsupportedEncodingException {
		return new String(data, pos, len);
	}

	public static byte[] selectBytes(byte[] data, int pos, int len)
		throws UnsupportedEncodingException {
		byte[] res = new byte[len];
		for (int i = 0; i < len; i++)
			res[i] = data[pos + i];
		return res;
	}

	/**
	 * begin of the code stolen from com.linuxnet.jpcsc.Apdu.java
	 * ----------------------------------
	 */

	/**
	 * Output format: hex space hex space hex
	 */
	public static final int HEX_SPACE_HEX_FORMAT = 2;

	/**
	 * Output format: hex:hex:hex
	 */
	public static final int HEX_COLON_HEX_FORMAT = 0;

	/**
	 * Output format: hexhexhex
	 */
	public static final int HEX_VOID_HEX_FORMAT = 1;

	/**
	 * The Format class described the format to use for converting APDU strings
	 * to buffers and vice versa.
	 */
	public static class Format {
		/**
		 * Format of data printed.
		 */
		public int dataFormat;

		/**
		 * Prepend string with APDU length.
		 */
		public boolean printLength;

		/**
		 * Append ASCII representation of APDU.
		 */
		public boolean printAscii;

		/**
		 * Constructor.
		 */
		public Format() {
			dataFormat = HEX_VOID_HEX_FORMAT;
			printLength = true;
			printAscii = false;
		}

		/**
		 * Constructor.
		 */
		public Format(int dataFormat, boolean printLength, boolean printAscii) {
			this.dataFormat = dataFormat;
			this.printLength = printLength;
			this.printAscii = printAscii;
		}
	}

	/**
	 * Default format for convert routines.
	 */
	private static Format defaultFormat = new Format();

	/**
	 * Helper string.
	 */
	private static String digits = "0123456789ABCDEF";

	/**
	 * Return string representation for byte array.
	 */
	public static String ba2s(byte[] ba) {
		return ba2s(ba, 0, ba.length, defaultFormat);
	}

	/**
	 * Return string representation for byte array range.
	 */
	public static String ba2s(byte[] ba, int off, int len) {
		return ba2s(ba, off, len, defaultFormat);
	}

	/**
	 * Return string representation for byte array.
	 */
	public static String ba2s(byte[] ba, Format f) {
		return ba2s(ba, 0, ba.length, f);
	}

	/**
	 * Return string representation for byte array range according to given
	 * format.
	 */
	public static String ba2s(byte[] ba, int off, int len, Format f) {
		StringBuffer sb = new StringBuffer();
		if (f.printLength) {
			sb.append(len).append(": ");
		}
		for (int i = off; i < off + len; i++) {
			int b = (int) ba[i];
			char c = digits.charAt((b >> 4) & 0xf);
			sb.append(c);
			c = digits.charAt(b & 0xf);
			sb.append(c);
			switch (f.dataFormat) {
			case HEX_SPACE_HEX_FORMAT:
				sb.append(' ');
				break;
			case HEX_COLON_HEX_FORMAT:
				sb.append(':');
				break;
			case HEX_VOID_HEX_FORMAT:
			default:
				break;
			}
		}
		if (f.printAscii) {
			sb.append(' ');
			for (int i = off; i < off + len; i++) {
				char c = (char) ba[i];
				if (Character.isLetterOrDigit(c)) {
					sb.append(c);
				} else {
					sb.append('X');
				}
			}
		}
		return sb.toString();
	}

	/**
	 * Convert string to a byte array. Notations supported are 0x00:0xb2 or
	 * 0x0:0xb2 or 00:b2 or 00:ba:|symbolic|:... or 00b2 or a0b0|symbolic|a0b0
	 * or 0x00 0xb2 or 0x0 0xb2 or 0x0 0xb2 |symbolic| ...
	 */
	public static byte[] s2ba(String s) {
		byte[] ba = new byte[s.length() * 2];
		int len = s2ba(s, ba, 0);
		byte[] ret = new byte[len];
		System.arraycopy(ba, 0, ret, 0, len);
		return ret;
	}

	/**
	 * Number of parentheses allowed in string specifications of APDUs.
	 */
	private static int MAX_BRACE_CNT = 10;

	/**
	 * Convert string to a byte array. Notations supported are 0x00:0xb2 or
	 * 0x0:0xb2 or 00:b2 or 00:ba for data bytes. Additionally, | can be used to
	 * insert ASCII characters into the byte stream, e.g. 01|abc|01 ->
	 * 0161626301 Another operator supported is #(...) where at the position of #
	 * the length of the byte string between ( and ) is inserted. Examples:
	 * #(5566) -> 025566, #(3344)#(55) -> 0233440155, #(11#(22)) -> 03110122,
	 * #(11#(|ZZ|)) -> 0411025A5A.
	 */
	public static int s2ba(String s, byte[] ba, int off) {
		int[] braceTab = new int[MAX_BRACE_CNT];
		int bracePos = -1;
		boolean asciiMode = false;
		int slen = s.length();
		String n;
		int bpos = off;
		int i = 0;
		byte b;

		while (i < slen) {
			char c = s.charAt(i);
			if (!asciiMode && (c == ':')) {
				i++;
				continue;
			}
			if (!asciiMode && (c == ' ')) {
				while (s.charAt(++i) == ' ')
					;
				continue;
			}

			if (!asciiMode && (c == '#')) {
				++i;
				if (s.charAt(i) != '(')
					throw new RuntimeException("miss ( after # in " + s);
				if (bracePos + 1 >= MAX_BRACE_CNT)
					throw new RuntimeException("too many ( in APDU spec " + s);
				braceTab[++bracePos] = bpos++;
				++i;
				continue;
			}
			if (!asciiMode && (c == ')')) {
				if (bracePos < 0)
					throw new RuntimeException("invalid ) in APDU spec " + s);
				ba[braceTab[bracePos]] = (byte) (bpos - braceTab[bracePos] - 1);
				--bracePos;
				++i;
				continue;
			}

			if (c == '|') {
				asciiMode = !asciiMode;
				i++;
				continue;
			}

			if (asciiMode) {
				if (c >= 256)
					throw new RuntimeException(
							"invalid character in hex-string " + s);
				b = (byte) ((int) c & 0xff);
				ba[bpos++] = b;
				i++;
				continue;
			}

			try {
				if ((c == '0')
						&& ((s.charAt(i + 1) == 'x') || (s.charAt(i + 1) == 'X'))) {
					i++;
					c = s.charAt(i);
				}

				if ((c == 'x') || (c == 'X')) {
					i++;
					if ((s.length() == i + 1)
							|| ((s.charAt(i + 1) == ':') || (s.charAt(i + 1) == ' '))) {
						n = s.substring(i, i + 1);
						i++; // set i on :
					} else {
						n = s.substring(i, i + 2);
						i += 2; // set i on :
					}
				} else {
					n = s.substring(i, i + 2);
					i += 2;
				}

				ba[bpos++] = (byte) (Integer.parseInt(n, 16) & 0xff);

			} catch (Exception e) {
				throw new RuntimeException("invalid hexadecimal string " + s);
			}
		}

		if (bracePos >= 0)
			throw new RuntimeException("mismatched parentheses");

		return bpos - off;
	}

	/**
	 * end of the code stolen from com.linuxnet.jpcsc.Apdu.java
	 * ----------------------------------
	 */

	public static void main(String[] argv) {
		try {
			throw new Exception(
					"throwing a (harmless) exception from the test suite");
		} catch (Exception e) {
			e.printStackTrace();
		}
		System.err.println("Test suites to be implemented...");
		System.err.println("No unexpected problems have been discovered...");
	}
}